@@ -24,12 +24,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 // KDE
 #include <kdebug.h>
+#include <kjobuidelegate.h>
 #include <kio/netaccess.h>
 #include <qtest_kde.h>
 
 // Local
 #include "../lib/abstractimageoperation.h"
 #include "../lib/document/abstractdocumenteditor.h"
+#include "../lib/document/documentjob.h"
 #include "../lib/document/documentfactory.h"
 #include "../lib/imagemetainfomodel.h"
 #include "../lib/imageutils.h"
@@ -52,6 +54,13 @@ static void waitUntilMetaInfoLoaded(Document::Ptr doc) {
 }
 
 
+static bool waitUntilJobIsDone(DocumentJob* job) {
+	JobWatcher watcher(job);
+	watcher.wait();
+	return watcher.error() == KJob::NoError;
+}
+
+
 void DocumentTest::initTestCase() {
 	qRegisterMetaType<KUrl>("KUrl");
 }
@@ -77,7 +86,7 @@ void DocumentTest::testLoad() {
 
 	Document::Ptr doc = DocumentFactory::instance()->load(url);
 	QSignalSpy spy(doc.data(), SIGNAL(isAnimatedUpdated()));
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 	QCOMPARE(doc->loadingState(), Document::Loaded);
 
@@ -133,7 +142,7 @@ void DocumentTest::testLoadTwoPasses() {
 	waitUntilMetaInfoLoaded(doc);
 	QVERIFY2(doc->image().isNull(), "Image shouldn't have been loaded at this time");
 	QCOMPARE(doc->format().data(), "png");
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 	QCOMPARE(image, doc->image());
 }
@@ -217,7 +226,7 @@ void DocumentTest::testLoadRemote() {
 	QVERIFY2(KIO::NetAccess::exists(url, KIO::NetAccess::SourceSide, 0), "test archive not found");
 
 	Document::Ptr doc = DocumentFactory::instance()->load(url);
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 	QImage image = doc->image();
 	QCOMPARE(image.width(), 300);
@@ -228,7 +237,7 @@ void DocumentTest::testLoadAnimated() {
 	KUrl srcUrl = urlForTestFile("40frames.gif");
 	Document::Ptr doc = DocumentFactory::instance()->load(srcUrl);
 	QSignalSpy spy(doc.data(), SIGNAL(imageRectUpdated(const QRect&)));
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 	QVERIFY(doc->isAnimated());
 
@@ -257,13 +266,13 @@ void DocumentTest::testLoadAnimated() {
 void DocumentTest::testSaveRemote() {
 	KUrl srcUrl = urlForTestFile("test.png");
 	Document::Ptr doc = DocumentFactory::instance()->load(srcUrl);
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 
 	KUrl dstUrl;
 	dstUrl.setProtocol("trash");
 	dstUrl.setPath("/test.png");
-	QVERIFY(!doc->save(dstUrl, "png"));
+	QVERIFY(!waitUntilJobIsDone(doc->save(dstUrl, "png")));
 
 	if (qgetenv("REMOTE_SFTP_TEST").isEmpty()) {
 		kWarning() << "*** Define the environment variable REMOTE_SFTP_TEST to try saving an image to sftp://localhost/tmp/test.png";
@@ -271,7 +280,7 @@ void DocumentTest::testSaveRemote() {
 		dstUrl.setProtocol("sftp");
 		dstUrl.setHost("localhost");
 		dstUrl.setPath("/tmp/test.png");
-		QVERIFY(doc->save(dstUrl, "png"));
+		QVERIFY(waitUntilJobIsDone(doc->save(dstUrl, "png")));
 	}
 }
 
@@ -300,7 +309,7 @@ void DocumentTest::testLoadRotated() {
 	image = image.transformed(matrix);
 
 	Document::Ptr doc = DocumentFactory::instance()->load(url);
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 	QCOMPARE(image, doc->image());
 }
@@ -324,10 +333,10 @@ void DocumentTest::testSaveAs() {
 	QSignalSpy savedSpy(doc.data(), SIGNAL(saved(const KUrl&, const KUrl&)));
 	QSignalSpy modifiedDocumentListChangedSpy(factory, SIGNAL(modifiedDocumentListChanged()));
 	QSignalSpy documentChangedSpy(factory, SIGNAL(documentChanged(const KUrl&)));
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 
 	KUrl destUrl = urlForTestOutputFile("result.png");
-	QVERIFY(doc->save(destUrl, "png"));
+	QVERIFY(waitUntilJobIsDone(doc->save(destUrl, "png")));
 	QCOMPARE(doc->format().data(), "png");
 	QCOMPARE(doc->url(), destUrl);
 
@@ -357,10 +366,10 @@ void DocumentTest::testSaveAs() {
 void DocumentTest::testLosslessSave() {
 	KUrl url1 = urlForTestFile("orient6.jpg");
 	Document::Ptr doc = DocumentFactory::instance()->load(url1);
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 
 	KUrl url2 = urlForTestOutputFile("orient1.jpg");
-	QVERIFY(doc->save(url2, "jpeg"));
+	QVERIFY(waitUntilJobIsDone(doc->save(url2, "jpeg")));
 
 	QImage image1;
 	QVERIFY(image1.load(url1.toLocalFile()));
@@ -387,7 +396,7 @@ void DocumentTest::testLosslessRotate() {
 
 	// Load it as a Gwenview document
 	Document::Ptr doc = DocumentFactory::instance()->load(url1);
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 
 	// Rotate one time
@@ -396,17 +405,17 @@ void DocumentTest::testLosslessRotate() {
 
 	// Save it
 	KUrl url2 = urlForTestOutputFile("lossless2.jpg");
-	doc->save(url2, "jpeg");
+	waitUntilJobIsDone(doc->save(url2, "jpeg"));
 
 	// Load the saved image
 	doc = DocumentFactory::instance()->load(url2);
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 
 	// Rotate the other way
 	QVERIFY(doc->editor());
 	doc->editor()->applyTransformation(ROT_270);
-	doc->save(url2, "jpeg");
+	waitUntilJobIsDone(doc->save(url2, "jpeg"));
 
 	// Compare the saved images
 	QVERIFY(image1.load(url1.toLocalFile()));
@@ -424,6 +433,7 @@ void DocumentTest::testModifyAndSaveAs() {
 			QImage image(10, 10, QImage::Format_ARGB32);
 			image.fill(QColor(Qt::white).rgb());
 			document()->editor()->setImage(image);
+			finish(true);
 		}
 	};
 	KUrl url = urlForTestFile("orient6.jpg");
@@ -434,7 +444,7 @@ void DocumentTest::testModifyAndSaveAs() {
 	QSignalSpy modifiedDocumentListChangedSpy(factory, SIGNAL(modifiedDocumentListChanged()));
 	QSignalSpy documentChangedSpy(factory, SIGNAL(documentChanged(const KUrl&)));
 
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 	QVERIFY(!doc->isModified());
 	QCOMPARE(modifiedDocumentListChangedSpy.count(), 0);
@@ -442,8 +452,7 @@ void DocumentTest::testModifyAndSaveAs() {
 	// Modify image
 	QVERIFY(doc->editor());
 	TestOperation* op = new TestOperation;
-	op->setDocument(doc);
-	doc->undoStack()->push(op);
+	op->applyToDocument(doc);
 	QVERIFY(doc->isModified());
 	QCOMPARE(modifiedDocumentListChangedSpy.count(), 1);
 	modifiedDocumentListChangedSpy.clear();
@@ -456,7 +465,7 @@ void DocumentTest::testModifyAndSaveAs() {
 
 	// Save it under a new name
 	KUrl destUrl = urlForTestOutputFile("modify.png");
-	QVERIFY(doc->save(destUrl, "png"));
+	QVERIFY(waitUntilJobIsDone(doc->save(destUrl, "png")));
 
 	// Wait a bit because save() will clear the undo stack when back to the
 	// event loop
@@ -533,13 +542,13 @@ void DocumentTest::testForgetModifiedDocument() {
 
 	// Load it as a Gwenview document
 	Document::Ptr doc = DocumentFactory::instance()->load(url);
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 
 	// Modify it
 	TransformImageOperation* op = new TransformImageOperation(ROT_90);
-	op->setDocument(doc);
-	doc->undoStack()->push(op);
+	op->applyToDocument(doc);
+	QTest::qWait(100);
 
 	QCOMPARE(spy.count(), 1);
 
@@ -563,20 +572,20 @@ void DocumentTest::testModifiedAndSavedSignals() {
 	Document::Ptr doc = DocumentFactory::instance()->load(url);
 	QSignalSpy modifiedSpy(doc.data(), SIGNAL(modified(const KUrl&)));
 	QSignalSpy savedSpy(doc.data(), SIGNAL(saved(const KUrl&, const KUrl&)));
-	doc->loadFullImage();
+	doc->startLoadingFullImage();
 	doc->waitUntilLoaded();
 
 	QCOMPARE(modifiedSpy.count(), 0);
 	QCOMPARE(savedSpy.count(), 0);
 
 	op = new TransformImageOperation(ROT_90);
-	op->setDocument(doc);
-	doc->undoStack()->push(op);
+	op->applyToDocument(doc);
+	QTest::qWait(100);
 	QCOMPARE(modifiedSpy.count(), 1);
 
 	op = new TransformImageOperation(ROT_90);
-	op->setDocument(doc);
-	doc->undoStack()->push(op);
+	op->applyToDocument(doc);
+	QTest::qWait(100);
 	QCOMPARE(modifiedSpy.count(), 2);
 
 	doc->undoStack()->undo();
@@ -585,3 +594,155 @@ void DocumentTest::testModifiedAndSavedSignals() {
 	doc->undoStack()->undo();
 	QCOMPARE(savedSpy.count(), 1);
 }
+
+class TestJob : public DocumentJob {
+public:
+	TestJob(QString* str, char ch)
+	: mStr(str)
+	, mCh(ch)
+	{}
+
+protected:
+	virtual void doStart() {
+		*mStr += mCh;
+		emitResult();
+	}
+
+private:
+	QString* mStr;
+	char mCh;
+};
+
+void DocumentTest::testJobQueue() {
+	KUrl url = urlForTestFile("orient6.jpg");
+	Document::Ptr doc = DocumentFactory::instance()->load(url);
+	QSignalSpy spy(doc.data(), SIGNAL(busyChanged(const KUrl&, bool)));
+
+	QString str;
+	doc->enqueueJob(new TestJob(&str, 'a'));
+	doc->enqueueJob(new TestJob(&str, 'b'));
+	doc->enqueueJob(new TestJob(&str, 'c'));
+	QVERIFY(doc->isBusy());
+	QEventLoop loop;
+	connect(doc.data(), SIGNAL(allTasksDone()),
+		&loop, SLOT(quit()));
+	loop.exec();
+	QVERIFY(!doc->isBusy());
+	QCOMPARE(spy.count(), 2);
+	QVariantList row = spy.takeFirst();
+	QCOMPARE(row.at(0).value<KUrl>(), url);
+	QVERIFY(row.at(1).toBool());
+	row = spy.takeFirst();
+	QCOMPARE(row.at(0).value<KUrl>(), url);
+	QVERIFY(!row.at(1).toBool());
+	QCOMPARE(str, QString("abc"));
+}
+
+
+class TestCheckDocumentEditorJob : public DocumentJob {
+public:
+	TestCheckDocumentEditorJob(int* hasEditor)
+	: mHasEditor(hasEditor)
+	{
+		*mHasEditor = -1;
+	}
+
+protected:
+	virtual void doStart() {
+		document()->waitUntilLoaded();
+		*mHasEditor = checkDocumentEditor() ? 1 : 0;
+		emitResult();
+	}
+
+private:
+	int* mHasEditor;
+};
+
+class TestUiDelegate : public KJobUiDelegate {
+public:
+	TestUiDelegate(bool* showErrorMessageCalled)
+	: mShowErrorMessageCalled(showErrorMessageCalled)
+	{
+		setAutoErrorHandlingEnabled(true);
+		*mShowErrorMessageCalled = false;
+	}
+
+	virtual void showErrorMessage() {
+		kDebug();
+		*mShowErrorMessageCalled = true;
+	}
+
+private:
+	bool* mShowErrorMessageCalled;
+};
+
+/**
+ * Test that an error is reported when a DocumentJob fails because there is no
+ * document editor available
+ */
+void DocumentTest::testCheckDocumentEditor() {
+	int hasEditor;
+	bool showErrorMessageCalled;
+	QEventLoop loop;
+	Document::Ptr doc;
+	TestCheckDocumentEditorJob* job;
+
+	doc = DocumentFactory::instance()->load(urlForTestFile("orient6.jpg"));
+
+	job = new TestCheckDocumentEditorJob(&hasEditor);
+	job->setUiDelegate(new TestUiDelegate(&showErrorMessageCalled));
+	doc->enqueueJob(job);
+	connect(doc.data(), SIGNAL(allTasksDone()), &loop, SLOT(quit()));
+	loop.exec();
+	QVERIFY(!showErrorMessageCalled);
+	QCOMPARE(hasEditor, 1);
+
+	doc = DocumentFactory::instance()->load(urlForTestFile("test.svg"));
+
+	job = new TestCheckDocumentEditorJob(&hasEditor);
+	job->setUiDelegate(new TestUiDelegate(&showErrorMessageCalled));
+	doc->enqueueJob(job);
+	connect(doc.data(), SIGNAL(allTasksDone()), &loop, SLOT(quit()));
+	loop.exec();
+	QVERIFY(showErrorMessageCalled);
+	QCOMPARE(hasEditor, 0);
+}
+
+
+/**
+ * An operation should only pushed to the document undo stack if it succeed
+ */
+void DocumentTest::testUndoStackPush() {
+	class SuccessOperation : public AbstractImageOperation {
+	protected:
+		virtual void redo() {
+			QMetaObject::invokeMethod(this, "finish", Qt::QueuedConnection, Q_ARG(bool, true));
+		}
+	};
+
+	class FailureOperation : public AbstractImageOperation {
+	protected:
+		virtual void redo() {
+			QMetaObject::invokeMethod(this, "finish", Qt::QueuedConnection, Q_ARG(bool, false));
+		}
+	};
+
+	AbstractImageOperation* op;
+	Document::Ptr doc = DocumentFactory::instance()->load(urlForTestFile("orient6.jpg"));
+
+	// A successful operation should be added to the undo stack
+	op = new SuccessOperation;
+	op->applyToDocument(doc);
+	QTest::qWait(100);
+	QVERIFY(!doc->undoStack()->isClean());
+
+	// Reset
+	doc->undoStack()->undo();
+	QVERIFY(doc->undoStack()->isClean());
+
+	// A failed operation should not be added to the undo stack
+	op = new FailureOperation;
+	op->applyToDocument(doc);
+	QTest::qWait(100);
+	QVERIFY(doc->undoStack()->isClean());
+}
