@@ -1,6 +1,6 @@
 /*
 Gwenview: an image viewer
-Copyright 2007 Aurélien Gâteau <aurelien.gateau@free.fr>
+Copyright 2007 Aurélien Gâteau <agateau@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -21,15 +21,35 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #define THUMBNAILLOADJOBTEST_H
 
 // Qt
+#include <QColor>
+#include <QHash>
 #include <QObject>
+#include <QSize>
+
+class SandBox {
+public:
+	SandBox();
+	void initDir();
+	void fill();
+	void createTestImage(const QString& name, int width, int height, const QColor& color);
+	void copyTestImage(const QString& name, int width, int height);
+
+	QHash<QString, QSize> mSizeHash;
+	QString mPath;
+};
 
 class ThumbnailLoadJobTest : public QObject {
 	Q_OBJECT
 
 private Q_SLOTS:
 	void init();
+	void initTestCase();
 	void testLoadLocal();
 	void testLoadRemote();
+	void testUseEmbeddedOrNot();
+
+private:
+	SandBox mSandBox;
 };
 
 #endif // THUMBNAILLOADJOBTEST_H
