@@ -1,21 +1,45 @@
 /* -*- C++ -*-
  * File: libraw_c_api.cpp
- * Copyright 2008-2009 LibRaw LLC (info@libraw.org)
+ * Copyright 2008-2010 LibRaw LLC (info@libraw.org)
  * Created: Sat Mar  8 , 2008
  *
  * LibRaw C interface 
+
+
+LibRaw is free software; you can redistribute it and/or modify
+it under the terms of the one of three licenses as you choose:
+
+1. GNU LESSER GENERAL PUBLIC LICENSE version 2.1
+   (See file LICENSE.LGPL provided in LibRaw distribution archive for details).
+
+2. COMMON DEVELOPMENT AND DISTRIBUTION LICENSE (CDDL) Version 1.0
+   (See file LICENSE.CDDL provided in LibRaw distribution archive for details).
+
+3. LibRaw Software License 27032010
+   (See file LICENSE.LibRaw.pdf provided in LibRaw distribution archive for details).
+
+
+
  */
 #include <errno.h>
 #include "libraw/libraw.h"
 
 #ifdef __cplusplus
+#include <new>
 extern "C" 
 {
 #endif
 
     libraw_data_t *libraw_init(unsigned int flags)
     {
-        LibRaw *ret = new LibRaw(flags);
+        LibRaw *ret;
+        try {
+            ret = new LibRaw(flags);
+        }
+        catch (std::bad_alloc)
+            {
+                return NULL;
+            }
         return &(ret->imgdata);
     }
 
@@ -37,6 +61,14 @@ extern "C"
         return ip->rotate_fuji_raw();
     }
 
+    void libraw_subtract_black(libraw_data_t* lr)
+    {
+        if(!lr) return;
+        LibRaw *ip = (LibRaw*) lr->parent_class;
+        ip->subtract_black();
+    }
+
+
     int libraw_add_masked_borders_to_bitmap(libraw_data_t* lr)
     {
         if(!lr) return EINVAL;
@@ -50,6 +82,12 @@ extern "C"
         LibRaw *ip = (LibRaw*) lr->parent_class;
         return ip->open_file(file);
     }
+    int libraw_open_file_ex(libraw_data_t* lr, const char *file,INT64 sz)
+    {
+        if(!lr) return EINVAL;
+        LibRaw *ip = (LibRaw*) lr->parent_class;
+        return ip->open_file(file,sz);
+    }
     int libraw_open_buffer(libraw_data_t* lr, void *buffer, size_t size)
     {
         if(!lr) return EINVAL;
@@ -110,7 +148,6 @@ extern "C"
         LibRaw *ip = (LibRaw*) lr->parent_class;
         return ip->adjust_sizes_info_only();
     }
-
     int  libraw_dcraw_document_mode_processing(libraw_data_t* lr)
     {
         if(!lr) return EINVAL;
@@ -150,6 +187,10 @@ extern "C"
         return ip->dcraw_make_mem_thumb(errc);
     }
 
+    void libraw_dcraw_clear_mem(libraw_processed_image_t* p)
+    {
+        LibRaw::dcraw_clear_mem(p);
+    }
 #ifdef __cplusplus
 }
 #endif
