@@ -7,8 +7,8 @@
  * Description : Exiv2 library interface for KDE
  *               Exif manipulation methods
  *
- * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * Copyright (C) 2006-2010 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2010 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -22,9 +22,13 @@
  *
  * ============================================================ */
 
+// KDE includes
+
+#include <klocale.h>
+
 // Local includes.
 
-#include "kexiv2private.h"
+#include "kexiv2_p.h"
 #include "kexiv2.h"
 
 namespace KExiv2Iface
@@ -40,10 +44,10 @@ bool KExiv2::canWriteExif(const QString& filePath)
         Exiv2::AccessMode mode = image->checkMode(Exiv2::mdExif);
         return (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite);
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         std::string s(e.what());
-        kDebug(51003) << "Cannot check Exif access mode using Exiv2 (Error #" 
+        kDebug(51003) << "Cannot check Exif access mode using Exiv2 (Error #"
                       << e.code() << ": " << s.c_str() << ")" << endl;
     }
 
@@ -52,17 +56,17 @@ bool KExiv2::canWriteExif(const QString& filePath)
 
 bool KExiv2::hasExif() const
 {
-    return !d->exifMetadata.empty();
+    return !d->exifMetadata().empty();
 }
 
 bool KExiv2::clearExif() const
 {
     try
     {
-        d->exifMetadata.clear();
+        d->exifMetadata().clear();
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot clear Exif data using Exiv2 ", e);
     }
@@ -71,13 +75,18 @@ bool KExiv2::clearExif() const
 }
 
 QByteArray KExiv2::getExif(bool addExifHeader) const
+{
+    return getExifEncoded(addExifHeader);
+}
+
+QByteArray KExiv2::getExifEncoded(bool addExifHeader) const
 {
     try
     {
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
             QByteArray data;
-            Exiv2::ExifData& exif = d->exifMetadata;
+            Exiv2::ExifData& exif = d->exifMetadata();
 #if (EXIV2_TEST_VERSION(0,17,91))
             Exiv2::Blob blob;
             Exiv2::ExifParser::encode(blob, Exiv2::bigEndian, exif);
@@ -100,7 +109,7 @@ QByteArray KExiv2::getExif(bool addExifHeader) const
             return data;
         }
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         if (!d->filePath.isEmpty())
             kDebug(51003) << "From file " << d->filePath.toAscii().constData() << endl;
@@ -118,17 +127,17 @@ bool KExiv2::setExif(const QByteArray& data) const
         if (!data.isEmpty())
         {
 #if (EXIV2_TEST_VERSION(0,17,91))
-            Exiv2::ExifParser::decode(d->exifMetadata, (const Exiv2::byte*)data.data(), data.size());
-            return (!d->exifMetadata.empty());
+            Exiv2::ExifParser::decode(d->exifMetadata(), (const Exiv2::byte*)data.data(), data.size());
+            return (!d->exifMetadata().empty());
 #else
-            if (d->exifMetadata.load((const Exiv2::byte*)data.data(), data.size()) != 0)
+            if (d->exifMetadata().load((const Exiv2::byte*)data.data(), data.size()) != 0)
                 return false;
             else
                 return true;
 #endif
         }
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         if (!d->filePath.isEmpty())
             kDebug(51003) << "From file " << d->filePath.toAscii().constData() << endl;
@@ -141,12 +150,12 @@ bool KExiv2::setExif(const QByteArray& data) const
 
 KExiv2::MetaDataMap KExiv2::getExifTagsDataList(const QStringList &exifKeysFilter, bool invertSelection) const
 {
-    if (d->exifMetadata.empty())
+    if (d->exifMetadata().empty())
        return MetaDataMap();
 
     try
     {
-        Exiv2::ExifData exifData = d->exifMetadata;
+        Exiv2::ExifData exifData = d->exifMetadata();
         exifData.sortByKey();
 
         QString     ifDItemName;
@@ -162,6 +171,10 @@ KExiv2::MetaDataMap KExiv2::getExifTagsDataList(const QStringList &exifKeysFilte
             {
                 tagValue = d->convertCommentValue(*md);
             }
+            else if (key == "Exif.Image.0x935c")
+            {
+                tagValue = i18n("Data of size %1", md->value().size());
+            }
             else
             {
                 std::ostringstream os;
@@ -200,10 +213,10 @@ QString KExiv2::getExifComment() const
 {
     try
     {
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
             Exiv2::ExifKey key("Exif.Photo.UserComment");
-            Exiv2::ExifData exifData(d->exifMetadata);
+            Exiv2::ExifData exifData(d->exifMetadata());
             Exiv2::ExifData::iterator it = exifData.findKey(key);
 
             if (it != exifData.end())
@@ -216,7 +229,7 @@ QString KExiv2::getExifComment() const
             }
         }
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot find Exif User Comment using Exiv2 ", e);
     }
@@ -231,35 +244,44 @@ bool KExiv2::setExifComment(const QString& comment, bool setProgramName) const
 
     try
     {
-        if (comment.isEmpty())
-            return false;
+        removeExifTag("Exif.Photo.UserComment");
 
-    // Write as Unicode only when necessary.
-    QTextCodec *latin1Codec = QTextCodec::codecForName("iso8859-1");
-    if (latin1Codec->canEncode(comment))
-    {
-        // write as ASCII
-        std::string exifComment("charset=\"Ascii\" ");
-        exifComment += comment.toLatin1().constData();
-        d->exifMetadata["Exif.Photo.UserComment"] = exifComment;
-    }
-    else
-    {
-        // write as Unicode (UCS-2)
+        if (!comment.isNull())
+        {
+            // Write as Unicode only when necessary.
+            QTextCodec *latin1Codec = QTextCodec::codecForName("iso8859-1");
+            if (latin1Codec->canEncode(comment))
+            {
+                // write as ASCII
+                std::string exifComment("charset=\"Ascii\" ");
+                exifComment += comment.toLatin1().constData();
+                d->exifMetadata()["Exif.Photo.UserComment"] = exifComment;
+            }
+            else
+            {
+                // write as Unicode (UCS-2)
 
-        // Be aware that we are dealing with a UCS-2 string.
-        // Null termination means \0\0, strlen does not work,
-        // do not use any const-char*-only methods,
-        // pass a std::string and not a const char * to ExifDatum::operator=().
-        const unsigned short *ucs2 = comment.utf16();
-        std::string exifComment("charset=\"Unicode\" ");
-        exifComment.append((const char*)ucs2, sizeof(unsigned short) * comment.length());
-        d->exifMetadata["Exif.Photo.UserComment"] = exifComment;
-    }
+#if (EXIV2_TEST_VERSION(0,20,0))
+                std::string exifComment("charset=\"Unicode\" ");
+                exifComment += comment.toUtf8().constData();
+                d->exifMetadata()["Exif.Photo.UserComment"] = exifComment;
+#else
+                // Older versions took a UCS2-String, see bug #205824
 
+                // Be aware that we are dealing with a UCS-2 string.
+                // Null termination means \0\0, strlen does not work,
+                // do not use any const-char*-only methods,
+                // pass a std::string and not a const char * to ExifDatum::operator=().
+                const unsigned short *ucs2 = comment.utf16();
+                std::string exifComment("charset=\"Unicode\" ");
+                exifComment.append((const char*)ucs2, sizeof(unsigned short) * comment.length());
+                d->exifMetadata()["Exif.Photo.UserComment"] = exifComment;
+#endif
+            }
+        }
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Exif Comment using Exiv2 ", e);
     }
@@ -267,15 +289,19 @@ bool KExiv2::setExifComment(const QString& comment, bool setProgramName) const
     return false;
 }
 
-QString KExiv2::getExifTagTitle(const char *exifTagName)
+QString KExiv2::getExifTagTitle(const char* exifTagName)
 {
-    try 
+    try
     {
         std::string exifkey(exifTagName);
-        Exiv2::ExifKey ek(exifkey); 
+        Exiv2::ExifKey ek(exifkey);
+#if (EXIV2_TEST_VERSION(0,21,0))
+        return QString::fromLocal8Bit( ek.tagLabel().c_str() );
+#else
         return QString::fromLocal8Bit( Exiv2::ExifTags::tagTitle(ek.tag(), ek.ifdId()) );
+#endif
     }
-    catch (Exiv2::Error& e) 
+    catch (Exiv2::Error& e)
     {
         d->printExiv2ExceptionError("Cannot get metadata tag title using Exiv2 ", e);
     }
@@ -283,15 +309,19 @@ QString KExiv2::getExifTagTitle(const char *exifTagName)
     return QString();
 }
 
-QString KExiv2::getExifTagDescription(const char *exifTagName)
+QString KExiv2::getExifTagDescription(const char* exifTagName)
 {
-    try 
+    try
     {
         std::string exifkey(exifTagName);
-        Exiv2::ExifKey ek(exifkey); 
+        Exiv2::ExifKey ek(exifkey);
+#if (EXIV2_TEST_VERSION(0,21,0))
+        return QString::fromLocal8Bit( ek.tagDesc().c_str() );
+#else
         return QString::fromLocal8Bit( Exiv2::ExifTags::tagDesc(ek.tag(), ek.ifdId()) );
+#endif
     }
-    catch (Exiv2::Error& e) 
+    catch (Exiv2::Error& e)
     {
         d->printExiv2ExceptionError("Cannot get metadata tag description using Exiv2 ", e);
     }
@@ -299,22 +329,20 @@ QString KExiv2::getExifTagDescription(const char *exifTagName)
     return QString();
 }
 
-bool KExiv2::removeExifTag(const char *exifTagName, bool setProgramName) const
+bool KExiv2::removeExifTag(const char* exifTagName, bool setProgramName) const
 {
     if (!setProgramId(setProgramName))
         return false;
 
     try
-    {  
+    {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData::iterator it = d->exifMetadata.findKey(exifKey);
-        if (it != d->exifMetadata.end())
+        Exiv2::ExifData::iterator it = d->exifMetadata().findKey(exifKey);
+        if (it != d->exifMetadata().end())
         {
-            d->exifMetadata.erase(it);
+            d->exifMetadata().erase(it);
             return true;
         }
-
-        return true;
     }
     catch( Exiv2::Error &e )
     {
@@ -324,12 +352,12 @@ bool KExiv2::removeExifTag(const char *exifTagName, bool setProgramName) const
     return false;
 }
 
-bool KExiv2::getExifTagRational(const char *exifTagName, long int &num, long int &den, int component) const
+bool KExiv2::getExifTagRational(const char* exifTagName, long int& num, long int& den, int component) const
 {
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
@@ -340,21 +368,21 @@ bool KExiv2::getExifTagRational(const char *exifTagName, long int &num, long int
     }
     catch( Exiv2::Error &e )
     {
-        d->printExiv2ExceptionError(QString("Cannot find Exif Rational value from key '%1' " 
+        d->printExiv2ExceptionError(QString("Cannot find Exif Rational value from key '%1' "
                                          "into image using Exiv2 ").arg(exifTagName), e);
     }
 
     return false;
 }
 
-bool KExiv2::setExifTagLong(const char *exifTagName, long val, bool setProgramName) const
+bool KExiv2::setExifTagLong(const char* exifTagName, long val, bool setProgramName) const
 {
     if (!setProgramId(setProgramName))
         return false;
 
     try
     {
-        d->exifMetadata[exifTagName] = static_cast<int32_t>(val);
+        d->exifMetadata()[exifTagName] = static_cast<int32_t>(val);
         return true;
     }
     catch( Exiv2::Error &e )
@@ -365,14 +393,14 @@ bool KExiv2::setExifTagLong(const char *exifTagName, long val, bool setProgramNa
     return false;
 }
 
-bool KExiv2::setExifTagRational(const char *exifTagName, long int num, long int den, bool setProgramName) const
+bool KExiv2::setExifTagRational(const char* exifTagName, long int num, long int den, bool setProgramName) const
 {
     if (!setProgramId(setProgramName))
         return false;
 
     try
     {
-        d->exifMetadata[exifTagName] = Exiv2::Rational(num, den);
+        d->exifMetadata()[exifTagName] = Exiv2::Rational(num, den);
         return true;
     }
     catch( Exiv2::Error &e )
@@ -383,7 +411,7 @@ bool KExiv2::setExifTagRational(const char *exifTagName, long int num, long int
     return false;
 }
 
-bool KExiv2::setExifTagData(const char *exifTagName, const QByteArray& data, bool setProgramName) const
+bool KExiv2::setExifTagData(const char* exifTagName, const QByteArray& data, bool setProgramName) const
 {
     if (data.isEmpty())
         return false;
@@ -394,7 +422,7 @@ bool KExiv2::setExifTagData(const char *exifTagName, const QByteArray& data, boo
     try
     {
         Exiv2::DataValue val((Exiv2::byte *)data.data(), data.size());
-        d->exifMetadata[exifTagName] = val;
+        d->exifMetadata()[exifTagName] = val;
         return true;
     }
     catch( Exiv2::Error &e )
@@ -405,7 +433,7 @@ bool KExiv2::setExifTagData(const char *exifTagName, const QByteArray& data, boo
     return false;
 }
 
-bool KExiv2::setExifTagVariant(const char *exifTagName, const QVariant& val, 
+bool KExiv2::setExifTagVariant(const char* exifTagName, const QVariant& val,
                                bool rationalWantSmallDenominator, bool setProgramName) const
 {
     switch (val.type())
@@ -450,7 +478,7 @@ bool KExiv2::setExifTagVariant(const char *exifTagName, const QVariant& val,
             try
             {
                 const std::string &exifdatetime(dateTime.toString(QString("yyyy:MM:dd hh:mm:ss")).toAscii().constData());
-                d->exifMetadata[exifTagName] = exifdatetime;
+                d->exifMetadata()[exifTagName] = exifdatetime;
             }
             catch( Exiv2::Error &e )
             {
@@ -471,7 +499,7 @@ bool KExiv2::setExifTagVariant(const char *exifTagName, const QVariant& val,
     return false;
 }
 
-QString KExiv2::createExifUserStringFromValue(const char *exifTagName, const QVariant &val, bool escapeCR)
+QString KExiv2::createExifUserStringFromValue(const char* exifTagName, const QVariant& val, bool escapeCR)
 {
     try
     {
@@ -551,14 +579,14 @@ QString KExiv2::createExifUserStringFromValue(const char *exifTagName, const QVa
     return QString();
 }
 
-bool KExiv2::getExifTagLong(const char* exifTagName, long &val) const
+bool KExiv2::getExifTagLong(const char* exifTagName, long& val) const
 {
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
-        if (it != exifData.end())
+        if (it != exifData.end() && it->count() > 0)
         {
             val = it->toLong();
             return true;
@@ -578,7 +606,7 @@ QByteArray KExiv2::getExifTagData(const char* exifTagName) const
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
@@ -589,7 +617,7 @@ QByteArray KExiv2::getExifTagData(const char* exifTagName) const
             (*it).copy((Exiv2::byte*)s, exifData.byteOrder());
 #endif
             QByteArray data(s, (*it).size());
-            delete s;
+            delete[] s;
             return data;
         }
     }
@@ -602,12 +630,12 @@ QByteArray KExiv2::getExifTagData(const char* exifTagName) const
     return QByteArray();
 }
 
-QVariant KExiv2::getExifTagVariant(const char *exifTagName, bool rationalAsListOfInts, bool stringEscapeCR, int component) const
+QVariant KExiv2::getExifTagVariant(const char* exifTagName, bool rationalAsListOfInts, bool stringEscapeCR, int component) const
 {
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
@@ -682,7 +710,7 @@ QString KExiv2::getExifTagString(const char* exifTagName, bool escapeCR) const
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
@@ -710,14 +738,14 @@ QString KExiv2::getExifTagString(const char* exifTagName, bool escapeCR) const
     return QString();
 }
 
-bool KExiv2::setExifTagString(const char *exifTagName, const QString& value, bool setProgramName) const
+bool KExiv2::setExifTagString(const char* exifTagName, const QString& value, bool setProgramName) const
 {
     if (!setProgramId(setProgramName))
         return false;
 
     try
     {
-        d->exifMetadata[exifTagName] = std::string(value.toAscii().constData());
+        d->exifMetadata()[exifTagName] = std::string(value.toAscii().constData());
         return true;
     }
     catch( Exiv2::Error &e )
@@ -732,16 +760,16 @@ QImage KExiv2::getExifThumbnail(bool fixOrientation) const
 {
     QImage thumbnail;
 
-    if (d->exifMetadata.empty())
+    if (d->exifMetadata().empty())
        return thumbnail;
 
     try
     {
 #if (EXIV2_TEST_VERSION(0,17,91))
-        Exiv2::ExifThumbC thumb(d->exifMetadata);
+        Exiv2::ExifThumbC thumb(d->exifMetadata());
         Exiv2::DataBuf const c1 = thumb.copy();
 #else
-        Exiv2::DataBuf const c1(d->exifMetadata.copyThumbnail());
+        Exiv2::DataBuf const c1(d->exifMetadata().copyThumbnail());
 #endif
         thumbnail.loadFromData(c1.pData_, c1.size_);
 
@@ -749,9 +777,12 @@ QImage KExiv2::getExifThumbnail(bool fixOrientation) const
         {
             if (fixOrientation)
             {
-                Exiv2::ExifKey key("Exif.Thumbnail.Orientation");
-                Exiv2::ExifData exifData(d->exifMetadata);
-                Exiv2::ExifData::iterator it = exifData.findKey(key);
+                Exiv2::ExifKey key1("Exif.Thumbnail.Orientation");
+                Exiv2::ExifKey key2("Exif.Image.Orientation");
+                Exiv2::ExifData exifData(d->exifMetadata());
+                Exiv2::ExifData::iterator it = exifData.findKey(key1);
+                if (it == exifData.end())
+                    it = exifData.findKey(key2);
                 if (it != exifData.end())
                 {
                     long orientation = it->toLong();
@@ -763,7 +794,7 @@ QImage KExiv2::getExifThumbnail(bool fixOrientation) const
             }
         }
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot get Exif Thumbnail using Exiv2 ", e);
     }
@@ -771,7 +802,7 @@ QImage KExiv2::getExifThumbnail(bool fixOrientation) const
     return thumbnail;
 }
 
-bool KExiv2::rotateExifQImage(QImage &image, ImageOrientation orientation) const
+bool KExiv2::rotateExifQImage(QImage& image, ImageOrientation orientation) const
 {
     QMatrix matrix;
 
@@ -820,7 +851,7 @@ bool KExiv2::rotateExifQImage(QImage &image, ImageOrientation orientation) const
     return false;
 }
 
-bool KExiv2::setExifThumbnail(const QImage& thumb, bool setProgramName) const
+bool KExiv2::setExifThumbnail(const QImage& thumbImage, bool setProgramName) const
 {
     if (!setProgramId(setProgramName))
         return false;
@@ -831,8 +862,8 @@ bool KExiv2::setExifThumbnail(const QImage& thumb, bool setProgramName) const
         QByteArray data;
         QBuffer buffer(&data);
         buffer.open(QIODevice::WriteOnly);
-        thumb.save(&buffer, "JPEG");
-        Exiv2::ExifThumb thumb(d->exifMetadata);
+        thumbImage.save(&buffer, "JPEG");
+        Exiv2::ExifThumb thumb(d->exifMetadata());
         thumb.setJpegThumbnail((Exiv2::byte *)data.data(), data.size());
 #else
         KTemporaryFile thumbFile;
@@ -842,11 +873,11 @@ bool KExiv2::setExifThumbnail(const QImage& thumb, bool setProgramName) const
         thumb.save(thumbFile.fileName(), "JPEG");
         kDebug(51003) << "Thumbnail temp file: " << thumbFile.fileName().toAscii().data() << endl;
         const std::string &fileName((const char*)(QFile::encodeName(thumbFile.fileName())));
-        d->exifMetadata.setJpegThumbnail( fileName );
+        d->exifMetadata().setJpegThumbnail( fileName );
 #endif
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Exif Thumbnail using Exiv2 ", e);
     }
@@ -854,4 +885,257 @@ bool KExiv2::setExifThumbnail(const QImage& thumb, bool setProgramName) const
     return false;
 }
 
+bool KExiv2::setTiffThumbnail(const QImage& thumbImage, bool setProgramName) const
+{
+    if (!setProgramId(setProgramName))
+        return false;
+
+    removeExifThumbnail();
+
+    try
+    {
+#if (EXIV2_TEST_VERSION(0,17,91))
+        // Make sure IFD0 is explicitely marked as a main image
+        Exiv2::ExifData::const_iterator pos = d->exifMetadata().findKey(Exiv2::ExifKey("Exif.Image.NewSubfileType"));
+        if (pos == d->exifMetadata().end() || pos->count() != 1 || pos->toLong() != 0) {
+            throw Exiv2::Error(1, "Exif.Image.NewSubfileType missing or not set as main image");
+        }
+        // Remove sub-IFD tags
+        std::string subImage1("SubImage1");
+        for (Exiv2::ExifData::iterator md = d->exifMetadata().begin(); md != d->exifMetadata().end();)
+        {
+            if (md->groupName() == subImage1)
+                md = d->exifMetadata().erase(md);
+            else
+                ++md;
+        }
+        // Set thumbnail tags
+        QByteArray data;
+        QBuffer buffer(&data);
+        buffer.open(QIODevice::WriteOnly);
+        thumbImage.save(&buffer, "JPEG");
+
+        Exiv2::DataBuf buf((Exiv2::byte *)data.data(), data.size());
+        Exiv2::ULongValue val;
+        val.read("0");
+        val.setDataArea(buf.pData_, buf.size_);
+        d->exifMetadata()["Exif.SubImage1.JPEGInterchangeFormat"] = val;
+        d->exifMetadata()["Exif.SubImage1.JPEGInterchangeFormatLength"] = uint32_t(buf.size_);
+        d->exifMetadata()["Exif.SubImage1.Compression"] = uint16_t(6); // JPEG (old-style)
+        d->exifMetadata()["Exif.SubImage1.NewSubfileType"] = uint32_t(1); // Thumbnail image
+        return true;
+#endif
+    }
+    catch( Exiv2::Error& e )
+    {
+        d->printExiv2ExceptionError("Cannot set TIFF Thumbnail using Exiv2 ", e);
+    }
+
+    return false;
+}
+
+bool KExiv2::removeExifThumbnail() const
+{
+    try
+    {
+#if (EXIV2_TEST_VERSION(0,17,91))
+        // Remove all IFD0 subimages.
+        Exiv2::ExifThumb thumb(d->exifMetadata());
+        thumb.erase();
+        return true;
+#endif
+    }
+    catch( Exiv2::Error& e )
+    {
+        d->printExiv2ExceptionError("Cannot remove Exif Thumbnail using Exiv2 ", e);
+    }
+
+    return false;
+}
+
+KExiv2::TagsMap KExiv2::getStdExifTagsList() const
+{
+    try
+    {
+        QList<const Exiv2::TagInfo*> tags;
+        TagsMap                      tagsMap;
+
+#if (EXIV2_TEST_VERSION(0,21,0))
+        const Exiv2::GroupInfo* gi = Exiv2::ExifTags::groupList();
+        while (gi->tagList_ != 0)
+        {
+            if (QString(gi->ifdName_) != QString("Makernote"))
+            {
+                Exiv2::TagListFct tl     = gi->tagList_;
+                const Exiv2::TagInfo* ti = tl();
+
+                while (ti->tag_ != 0xFFFF)
+                {
+                    tags << ti;
+                    ++ti;
+                }
+            }
+            ++gi;
+        }
+
+        for (QList<const Exiv2::TagInfo*>::iterator it = tags.begin(); it != tags.end(); ++it)
+        {
+            do
+            {
+                const Exiv2::TagInfo* ti = *it;
+                QString key              = QLatin1String(Exiv2::ExifKey(*ti).key().c_str());
+                QStringList values;
+                values << ti->name_ << ti->title_ << ti->desc_;
+                tagsMap.insert(key, values);
+                ++(*it);
+            }
+            while((*it)->tag_ != 0xffff);
+        }
+#else
+        tags << Exiv2::ExifTags::ifdTagList()
+             << Exiv2::ExifTags::exifTagList()
+             << Exiv2::ExifTags::iopTagList()
+             << Exiv2::ExifTags::gpsTagList();
+
+        for (QList<const Exiv2::TagInfo*>::iterator it = tags.begin(); it != tags.end(); ++it)
+        {
+            do
+            {
+                QString key = QLatin1String( Exiv2::ExifKey( (*it)->tag_, Exiv2::ExifTags::ifdItem( (*it)->ifdId_ ) ).key().c_str() );
+                QStringList values;
+                values << (*it)->name_ << (*it)->title_ << (*it)->desc_;
+                tagsMap.insert(key, values);
+                ++(*it);
+            }
+            while((*it)->tag_ != 0xffff);
+        }
+#endif
+        return tagsMap;
+    }
+    catch( Exiv2::Error& e )
+    {
+        d->printExiv2ExceptionError("Cannot get Exif Tags list using Exiv2 ", e);
+    }
+
+    return TagsMap();
+}
+
+KExiv2::TagsMap KExiv2::getMakernoteTagsList() const
+{
+    try
+    {
+        QList<const Exiv2::TagInfo*> tags;
+        TagsMap                      tagsMap;
+
+#if (EXIV2_TEST_VERSION(0,21,0))
+
+        const Exiv2::GroupInfo* gi = Exiv2::ExifTags::groupList();
+
+        while (gi->tagList_ != 0)
+        {
+            if (QString(gi->ifdName_) == QString("Makernote"))
+            {
+                Exiv2::TagListFct tl     = gi->tagList_;
+                const Exiv2::TagInfo* ti = tl();
+
+                while (ti->tag_ != 0xFFFF)
+                {
+                    tags << ti;
+                    ++ti;
+                }
+            }
+            ++gi;
+        }
+
+        for (QList<const Exiv2::TagInfo*>::iterator it = tags.begin(); it != tags.end(); ++it)
+        {
+            do
+            {
+                const Exiv2::TagInfo* ti = *it;
+                QString key              = QLatin1String(Exiv2::ExifKey(*ti).key().c_str());
+                QStringList values;
+                values << ti->name_ << ti->title_ << ti->desc_;
+                tagsMap.insert(key, values);
+                ++(*it);
+            }
+            while((*it)->tag_ != 0xffff);
+        }
+
+#else
+
+#if (EXIV2_TEST_VERSION(0,18,1))
+        tags
+             // Canon Makernotes.
+             << Exiv2::CanonMakerNote::tagList()
+             << Exiv2::CanonMakerNote::tagListCs()
+             << Exiv2::CanonMakerNote::tagListSi()
+             << Exiv2::CanonMakerNote::tagListPa()
+             << Exiv2::CanonMakerNote::tagListCf()
+             << Exiv2::CanonMakerNote::tagListPi()
+#if (EXIV2_TEST_VERSION(0,19,1))
+             << Exiv2::CanonMakerNote::tagListFi()
+#endif // (EXIV2_TEST_VERSION(0,19,1))
+             // Sigma Makernotes.
+             << Exiv2::SigmaMakerNote::tagList()
+             // Sony Makernotes.
+             << Exiv2::SonyMakerNote::tagList()
+#if (EXIV2_TEST_VERSION(0,19,1))
+             << Exiv2::SonyMakerNote::tagListCs()
+             << Exiv2::SonyMakerNote::tagListCs2()
+#endif // (EXIV2_TEST_VERSION(0,19,1))
+             // Minolta Makernotes.
+             << Exiv2::MinoltaMakerNote::tagList()
+             << Exiv2::MinoltaMakerNote::tagListCsStd()
+             << Exiv2::MinoltaMakerNote::tagListCs7D()
+             << Exiv2::MinoltaMakerNote::tagListCs5D()
+             // Nikon Makernotes.
+             << Exiv2::Nikon1MakerNote::tagList()
+             << Exiv2::Nikon2MakerNote::tagList()
+             << Exiv2::Nikon3MakerNote::tagList()
+             // Olympus Makernotes.
+             << Exiv2::OlympusMakerNote::tagList()
+             << Exiv2::OlympusMakerNote::tagListCs()
+             << Exiv2::OlympusMakerNote::tagListEq()
+             << Exiv2::OlympusMakerNote::tagListRd()
+             << Exiv2::OlympusMakerNote::tagListRd2()
+             << Exiv2::OlympusMakerNote::tagListIp()
+             << Exiv2::OlympusMakerNote::tagListFi()
+             << Exiv2::OlympusMakerNote::tagListFe()
+             << Exiv2::OlympusMakerNote::tagList()
+             << Exiv2::OlympusMakerNote::tagListRi()
+             // Panasonic Makernotes.
+             << Exiv2::PanasonicMakerNote::tagList()
+             << Exiv2::PanasonicMakerNote::tagListRaw()
+             // Pentax Makernotes.
+             << Exiv2::PentaxMakerNote::tagList()
+             // Fuji Makernotes.
+             << Exiv2::FujiMakerNote::tagList();
+
+#endif // (EXIV2_TEST_VERSION(0,18,1))
+
+        for (QList<const Exiv2::TagInfo*>::iterator it = tags.begin(); it != tags.end(); ++it)
+        {
+            do
+            {
+                QString     key = QLatin1String( Exiv2::ExifKey( (*it)->tag_, Exiv2::ExifTags::ifdItem( (*it)->ifdId_ ) ).key().c_str() );
+                QStringList values;
+                values << (*it)->name_ << (*it)->title_ << (*it)->desc_;
+                tagsMap.insert(key, values);
+                ++(*it);
+            }
+            while((*it)->tag_ != 0xffff);
+        }
+
+#endif // (EXIV2_TEST_VERSION(0,21,0))
+
+        return tagsMap;
+    }
+    catch( Exiv2::Error& e )
+    {
+        d->printExiv2ExceptionError("Cannot get Makernote Tags list using Exiv2 ", e);
+    }
+
+    return TagsMap();
+}
+
 }  // NameSpace KExiv2Iface
