@@ -7,8 +7,8 @@
  * Description : Exiv2 library interface for KDE
  *               Common metadata image information manipulation methods
  *
- * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * Copyright (C) 2006-2010 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2010 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -24,8 +24,9 @@
 
 // Local includes.
 
-#include "kexiv2private.h"
+#include "kexiv2_p.h"
 #include "kexiv2.h"
+#include "rotationmatrix.h"
 
 namespace KExiv2Iface
 {
@@ -41,28 +42,28 @@ bool KExiv2::setImageProgramId(const QString& program, const QString& version) c
         // Set program info into Exif.Image.ProcessingSoftware tag (only available with Exiv2 >= 0.14.0).
 
 #if (EXIV2_TEST_VERSION(0,14,0))
-        d->exifMetadata["Exif.Image.ProcessingSoftware"] = std::string(software.toAscii().constData());
+        d->exifMetadata()["Exif.Image.ProcessingSoftware"] = std::string(software.toAscii().constData());
 #endif
 
         // See B.K.O #142564: Check if Exif.Image.Software already exist. If yes, do not touch this tag.
 
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
-            Exiv2::ExifData exifData(d->exifMetadata);
+            Exiv2::ExifData exifData(d->exifMetadata());
             Exiv2::ExifKey key("Exif.Image.Software");
             Exiv2::ExifData::iterator it = exifData.findKey(key);
             if (it == exifData.end())
-                d->exifMetadata["Exif.Image.Software"] = std::string(software.toAscii().constData());
+                d->exifMetadata()["Exif.Image.Software"] = std::string(software.toAscii().constData());
         }
 
         // set program info into XMP tags.
 
 #ifdef _XMP_SUPPORT_
 
-        if (!d->xmpMetadata.empty())
+        if (!d->xmpMetadata().empty())
         {
             // Only create Xmp.xmp.CreatorTool if it do not exist.
-            Exiv2::XmpData xmpData(d->xmpMetadata);
+            Exiv2::XmpData xmpData(d->xmpMetadata());
             Exiv2::XmpKey key("Xmp.xmp.CreatorTool");
             Exiv2::XmpData::iterator it = xmpData.findKey(key);
             if (it == xmpData.end())
@@ -75,11 +76,11 @@ bool KExiv2::setImageProgramId(const QString& program, const QString& version) c
 
         // Set program info into IPTC tags.
 
-        d->iptcMetadata["Iptc.Application2.Program"]        = std::string(program.toAscii().constData());
-        d->iptcMetadata["Iptc.Application2.ProgramVersion"] = std::string(version.toAscii().constData());
+        d->iptcMetadata()["Iptc.Application2.Program"]        = std::string(program.toAscii().constData());
+        d->iptcMetadata()["Iptc.Application2.ProgramVersion"] = std::string(version.toAscii().constData());
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Program identity into image using Exiv2 ", e);
     }
@@ -95,7 +96,7 @@ QSize KExiv2::getImageDimensions() const
 
         // Try to get Exif.Photo tags
 
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifKey key("Exif.Photo.PixelXDimension");
         Exiv2::ExifData::iterator it = exifData.findKey(key);
         if (it != exifData.end())
@@ -168,7 +169,7 @@ QSize KExiv2::getImageDimensions() const
 #endif // _XMP_SUPPORT_
 
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot parse image dimensions tag using Exiv2 ", e);
     }
@@ -186,10 +187,10 @@ bool KExiv2::setImageDimensions(const QSize& size, bool setProgramName) const
         // Set Exif values.
 
         // NOTE: see B.K.O #144604: need to cast to record an unsigned integer value.
-        d->exifMetadata["Exif.Image.ImageWidth"]      = static_cast<uint32_t>(size.width());
-        d->exifMetadata["Exif.Image.ImageLength"]     = static_cast<uint32_t>(size.height());
-        d->exifMetadata["Exif.Photo.PixelXDimension"] = static_cast<uint32_t>(size.width());
-        d->exifMetadata["Exif.Photo.PixelYDimension"] = static_cast<uint32_t>(size.height());
+        d->exifMetadata()["Exif.Image.ImageWidth"]      = static_cast<uint32_t>(size.width());
+        d->exifMetadata()["Exif.Image.ImageLength"]     = static_cast<uint32_t>(size.height());
+        d->exifMetadata()["Exif.Photo.PixelXDimension"] = static_cast<uint32_t>(size.width());
+        d->exifMetadata()["Exif.Photo.PixelYDimension"] = static_cast<uint32_t>(size.height());
 
         // Set Xmp values.
 
@@ -204,7 +205,7 @@ bool KExiv2::setImageDimensions(const QSize& size, bool setProgramName) const
 
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set image dimensions using Exiv2 ", e);
     }
@@ -216,7 +217,7 @@ KExiv2::ImageOrientation KExiv2::getImageOrientation() const
 {
     try
     {
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it;
         long orientation;
         ImageOrientation imageOrient = ORIENTATION_NORMAL;
@@ -295,7 +296,7 @@ KExiv2::ImageOrientation KExiv2::getImageOrientation() const
 #endif // _XMP_SUPPORT_
 
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot parse Exif Orientation tag using Exiv2 ", e);
     }
@@ -318,7 +319,7 @@ bool KExiv2::setImageOrientation(ImageOrientation orientation, bool setProgramNa
 
         // Set Exif values.
 
-        d->exifMetadata["Exif.Image.Orientation"] = static_cast<uint16_t>(orientation);
+        d->exifMetadata()["Exif.Image.Orientation"] = static_cast<uint16_t>(orientation);
         kDebug(51003) << "Exif.Image.Orientation tag set to: " << (int)orientation << endl;
 
         // Set Xmp values.
@@ -329,32 +330,43 @@ bool KExiv2::setImageOrientation(ImageOrientation orientation, bool setProgramNa
 
 #endif // _XMP_SUPPORT_
 
-        // -- Minolta Cameras ----------------------------------
+        // -- Minolta/Sony Cameras ----------------------------------
 
-        // Minolta camera store image rotation in Makernote.
-        // We remove these information to prevent duplicate values. 
+        // Minolta and Sony camera store image rotation in Makernote.
+        // We remove these information to prevent duplicate values.
 
         Exiv2::ExifData::iterator it;
 
         Exiv2::ExifKey minoltaKey1("Exif.MinoltaCs7D.Rotation");
-        it = d->exifMetadata.findKey(minoltaKey1);
-        if (it != d->exifMetadata.end())
+        it = d->exifMetadata().findKey(minoltaKey1);
+        if (it != d->exifMetadata().end())
         {
-            d->exifMetadata.erase(it);
+            d->exifMetadata().erase(it);
             kDebug(51003) << "Removing Exif.MinoltaCs7D.Rotation tag" << endl;
         }
 
         Exiv2::ExifKey minoltaKey2("Exif.MinoltaCs5D.Rotation");
-        it = d->exifMetadata.findKey(minoltaKey2);
-        if (it != d->exifMetadata.end())
+        it = d->exifMetadata().findKey(minoltaKey2);
+        if (it != d->exifMetadata().end())
         {
-            d->exifMetadata.erase(it);
+            d->exifMetadata().erase(it);
             kDebug(51003) << "Removing Exif.MinoltaCs5D.Rotation tag" << endl;
         }
 
+        // -- Exif embedded thumbnail ----------------------------------
+
+        Exiv2::ExifKey thumbKey("Exif.Thumbnail.Orientation");
+        it = d->exifMetadata().findKey(thumbKey);
+        if (it != d->exifMetadata().end())
+        {
+            RotationMatrix operation((KExiv2Iface::KExiv2::ImageOrientation)it->toLong());
+            operation *= orientation;
+            (*it) = static_cast<uint16_t>(operation.exifOrientation());
+        }
+
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Exif Orientation tag using Exiv2 ", e);
     }
@@ -366,71 +378,71 @@ KExiv2::ImageColorWorkSpace KExiv2::getImageColorWorkSpace() const
 {
     // Check Exif values.
 
-    long colorSpace = 0;
-
-    if (getExifTagLong("Exif.Photo.ColorSpace", colorSpace))
+    long exifColorSpace = -1;
+    if (!getExifTagLong("Exif.Photo.ColorSpace", exifColorSpace))
     {
-        switch (colorSpace)
-        {
-            case 1:
-            {
-                return WORKSPACE_SRGB;
-                break;
-            }
-            case 2:
-            {
-                return WORKSPACE_ADOBERGB;
-                break;
-            }
-            case 65535:
-            {
-                // Nikon camera set Exif.Photo.ColorSpace to uncalibrated and 
-                // Exif.Nikon3.ColorMode to "MODE2" when users work in AdobRGB color space.
-                if (getExifTagString("Exif.Nikon3.ColorMode").contains("MODE2"))
-                    return WORKSPACE_ADOBERGB;
-
-                // TODO : add more Makernote parsing here ...
-
-                return WORKSPACE_UNCALIBRATED;
-                break;
-            }
-        }
+        #ifdef _XMP_SUPPORT_
+        QVariant var = getXmpTagVariant("Xmp.exif.ColorSpace");
+        if (!var.isNull())
+            exifColorSpace = var.toInt();
+        #endif // _XMP_SUPPORT_
     }
 
-    // Check Xmp values.
-
-#ifdef _XMP_SUPPORT_
-
-    colorSpace  = 0;
-    bool ok     = false;
-    QString str = getXmpTagString("Xmp.exif.ColorSpace");
-    if (!str.isEmpty())
+    if (exifColorSpace == 1)
+        return WORKSPACE_SRGB; // as specified by standard
+    else if (exifColorSpace == 2)
+        return WORKSPACE_ADOBERGB; // not in the standard!
+    else
     {
-        colorSpace = str.toLong(&ok);
-        if (ok)
+        if (exifColorSpace == 65535)
         {
-            switch (colorSpace)
+            // A lot of cameras set the Exif.Iop.InteroperabilityIndex,
+            // as documented for ExifTool
+            QString interopIndex = getExifTagString("Exif.Iop.InteroperabilityIndex");
+            if (!interopIndex.isNull())
             {
-                case 1:
-                {
+                if (interopIndex == "R03")
+                    return WORKSPACE_ADOBERGB;
+                else if (interopIndex == "R98")
                     return WORKSPACE_SRGB;
-                    break;
-                }
-                case 2:
-                {
-                    return WORKSPACE_ADOBERGB;
-                    break;
-                }
-                case 65535:
-                {
-                    return WORKSPACE_UNCALIBRATED;
-                    break;
-                }
             }
         }
-    }
 
-#endif // _XMP_SUPPORT_
+        // Note: Text EXIF ColorSpace tag may just not be present (NEF files)
+
+        // Nikon camera set Exif.Photo.ColorSpace to uncalibrated or just skip this field,
+        // then add additional information into the makernotes.
+        // Exif.Nikon3.ColorSpace: 1 => sRGB, 2 => AdobeRGB
+        long nikonColorSpace;
+        if (getExifTagLong("Exif.Nikon3.ColorSpace", nikonColorSpace))
+        {
+            if (nikonColorSpace == 1)
+                return WORKSPACE_SRGB;
+            else if (nikonColorSpace == 2)
+                return WORKSPACE_ADOBERGB;
+        }
+        // Exif.Nikon3.ColorMode is set to "MODE2" for AdobeRGB, but there are sometimes two ColorMode fields
+        // in a NEF, with the first one "COLOR" and the second one "MODE2"; but in this case, ColorSpace (above) was set.
+        if (getExifTagString("Exif.Nikon3.ColorMode").contains("MODE2"))
+            return WORKSPACE_ADOBERGB;
+
+        //TODO: This makernote tag (0x00b4) must be added to libexiv2
+        /*
+        long canonColorSpace;
+        if (getExifTagLong("Exif.Canon.ColorSpace", canonColorSpace))
+        {
+            if (canonColorSpace == 1)
+                return WORKSPACE_SRGB;
+            else if (canonColorSpace == 2)
+                return WORKSPACE_ADOBERGB;
+        }
+        */
+
+        // TODO : add more Makernote parsing here ...
+
+        if (exifColorSpace == 65535)
+            return WORKSPACE_UNCALIBRATED;
+    }
 
     return WORKSPACE_UNSPECIFIED;
 }
@@ -444,7 +456,7 @@ bool KExiv2::setImageColorWorkSpace(ImageColorWorkSpace workspace, bool setProgr
     {
         // Set Exif value.
 
-        d->exifMetadata["Exif.Photo.ColorSpace"] = static_cast<uint16_t>(workspace);
+        d->exifMetadata()["Exif.Photo.ColorSpace"] = static_cast<uint16_t>(workspace);
 
         // Set Xmp value.
 
@@ -456,7 +468,7 @@ bool KExiv2::setImageColorWorkSpace(ImageColorWorkSpace workspace, bool setProgr
 
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Exif color workspace tag using Exiv2 ", e);
     }
@@ -470,9 +482,9 @@ QDateTime KExiv2::getImageDateTime() const
     {
         // In first, trying to get Date & time from Exif tags.
 
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
-            Exiv2::ExifData exifData(d->exifMetadata);
+            Exiv2::ExifData exifData(d->exifMetadata());
             {
                 Exiv2::ExifKey key("Exif.Photo.DateTimeOriginal");
                 Exiv2::ExifData::iterator it = exifData.findKey(key);
@@ -518,9 +530,9 @@ QDateTime KExiv2::getImageDateTime() const
 
 #ifdef _XMP_SUPPORT_
 
-        if (!d->xmpMetadata.empty())
+        if (!d->xmpMetadata().empty())
         {
-            Exiv2::XmpData xmpData(d->xmpMetadata);
+            Exiv2::XmpData xmpData(d->xmpMetadata());
             {
                 Exiv2::XmpKey key("Xmp.exif.DateTimeOriginal");
                 Exiv2::XmpData::iterator it = xmpData.findKey(key);
@@ -618,9 +630,9 @@ QDateTime KExiv2::getImageDateTime() const
 
         // In third, trying to get Date & time from Iptc tags.
 
-        if (!d->iptcMetadata.empty())
+        if (!d->iptcMetadata().empty())
         {
-            Exiv2::IptcData iptcData(d->iptcMetadata);
+            Exiv2::IptcData iptcData(d->iptcMetadata());
 
             // Try creation Iptc date & time entries.
 
@@ -669,7 +681,7 @@ QDateTime KExiv2::getImageDateTime() const
             }
         }
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot parse Exif date & time tag using Exiv2 ", e);
     }
@@ -695,10 +707,10 @@ bool KExiv2::setImageDateTime(const QDateTime& dateTime, bool setDateTimeDigitiz
         // Reference: http://www.exif.org/Exif2-2.PDF, chapter 4.6.5, table 4, section F.
 
         const std::string &exifdatetime(dateTime.toString(QString("yyyy:MM:dd hh:mm:ss")).toAscii().constData());
-        d->exifMetadata["Exif.Image.DateTime"]         = exifdatetime;
-        d->exifMetadata["Exif.Photo.DateTimeOriginal"] = exifdatetime;
+        d->exifMetadata()["Exif.Image.DateTime"]         = exifdatetime;
+        d->exifMetadata()["Exif.Photo.DateTimeOriginal"] = exifdatetime;
         if(setDateTimeDigitized)
-            d->exifMetadata["Exif.Photo.DateTimeDigitized"] = exifdatetime;
+            d->exifMetadata()["Exif.Photo.DateTimeDigitized"] = exifdatetime;
 
 #ifdef _XMP_SUPPORT_
 
@@ -708,14 +720,14 @@ bool KExiv2::setImageDateTime(const QDateTime& dateTime, bool setDateTimeDigitiz
 
         Exiv2::Value::AutoPtr xmpTxtVal = Exiv2::Value::create(Exiv2::xmpText);
         xmpTxtVal->read(xmpdatetime);
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.exif.DateTimeOriginal"), xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.photoshop.DateCreated"), xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.tiff.DateTime"),         xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.xmp.CreateDate"),        xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.xmp.MetadataDate"),      xmpTxtVal.get());
-        d->xmpMetadata.add(Exiv2::XmpKey("Xmp.xmp.ModifyDate"),        xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.exif.DateTimeOriginal"), xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.photoshop.DateCreated"), xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.tiff.DateTime"),         xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.xmp.CreateDate"),        xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.xmp.MetadataDate"),      xmpTxtVal.get());
+        d->xmpMetadata().add(Exiv2::XmpKey("Xmp.xmp.ModifyDate"),        xmpTxtVal.get());
         if(setDateTimeDigitized)
-            d->xmpMetadata.add(Exiv2::XmpKey("Xmp.exif.DateTimeDigitized"), xmpTxtVal.get());
+            d->xmpMetadata().add(Exiv2::XmpKey("Xmp.exif.DateTimeDigitized"), xmpTxtVal.get());
 
         // Tag not updated:
         // "Xmp.dc.DateTime" is a sequence of date relevant of dublin core change. 
@@ -727,17 +739,17 @@ bool KExiv2::setImageDateTime(const QDateTime& dateTime, bool setDateTimeDigitiz
 
         const std::string &iptcdate(dateTime.date().toString(Qt::ISODate).toAscii().constData());
         const std::string &iptctime(dateTime.time().toString(Qt::ISODate).toAscii().constData());
-        d->iptcMetadata["Iptc.Application2.DateCreated"] = iptcdate;
-        d->iptcMetadata["Iptc.Application2.TimeCreated"] = iptctime;
+        d->iptcMetadata()["Iptc.Application2.DateCreated"] = iptcdate;
+        d->iptcMetadata()["Iptc.Application2.TimeCreated"] = iptctime;
         if(setDateTimeDigitized)
         {
-            d->iptcMetadata["Iptc.Application2.DigitizationDate"] = iptcdate;
-            d->iptcMetadata["Iptc.Application2.DigitizationTime"] = iptctime;
+            d->iptcMetadata()["Iptc.Application2.DigitizationDate"] = iptcdate;
+            d->iptcMetadata()["Iptc.Application2.DigitizationTime"] = iptctime;
         }
 
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot set Date & Time into image using Exiv2 ", e);
     }
@@ -751,11 +763,11 @@ QDateTime KExiv2::getDigitizationDateTime(bool fallbackToCreationTime) const
     {
         // In first, trying to get Date & time from Exif tags.
 
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
             // Try Exif date time digitized.
 
-            Exiv2::ExifData exifData(d->exifMetadata);
+            Exiv2::ExifData exifData(d->exifMetadata());
             Exiv2::ExifKey key("Exif.Photo.DateTimeDigitized");
             Exiv2::ExifData::iterator it = exifData.findKey(key);
 
@@ -774,12 +786,12 @@ QDateTime KExiv2::getDigitizationDateTime(bool fallbackToCreationTime) const
 
         // In second, trying to get Date & time from Iptc tags.
 
-        if (!d->iptcMetadata.empty())
+        if (!d->iptcMetadata().empty())
         {
 
             // Try digitization Iptc date time entries.
 
-            Exiv2::IptcData iptcData(d->iptcMetadata);
+            Exiv2::IptcData iptcData(d->iptcMetadata());
             Exiv2::IptcKey keyDigitizationDate("Iptc.Application2.DigitizationDate");
             Exiv2::IptcData::iterator it = iptcData.findKey(keyDigitizationDate);
 
@@ -807,7 +819,7 @@ QDateTime KExiv2::getDigitizationDateTime(bool fallbackToCreationTime) const
             }
         }
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot parse Exif digitization date & time tag using Exiv2 ", e);
     }
@@ -830,7 +842,7 @@ bool KExiv2::getImagePreview(QImage& preview) const
 
         // TODO : Added here Makernotes preview extraction when Exiv2 will be fixed for that.
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot get image preview using Exiv2 ", e);
     }
@@ -856,15 +868,15 @@ bool KExiv2::setImagePreview(const QImage& preview, bool setProgramName) const
 
         Exiv2::DataValue val;
         val.read((Exiv2::byte *)data.data(), data.size());
-        d->iptcMetadata["Iptc.Application2.Preview"] = val;
+        d->iptcMetadata()["Iptc.Application2.Preview"] = val;
 
         // See http://www.iptc.org/std/IIM/4.1/specification/IIMV4.1.pdf Appendix A for details.
-        d->iptcMetadata["Iptc.Application2.PreviewFormat"]  = 11;  // JPEG 
-        d->iptcMetadata["Iptc.Application2.PreviewVersion"] = 1;
+        d->iptcMetadata()["Iptc.Application2.PreviewFormat"]  = 11;  // JPEG
+        d->iptcMetadata()["Iptc.Application2.PreviewVersion"] = 1;
 
         return true;
     }
-    catch( Exiv2::Error &e )
+    catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot get image preview using Exiv2 ", e);
     }
