@@ -31,12 +31,15 @@
 #include "ksane.h"
 #include "ksane.moc"
 
+#include <unistd.h>
+
 // Qt includes
 #include <QApplication>
 #include <QVarLengthArray>
 #include <QLabel>
 #include <QSplitter>
 #include <QMutex>
+#include <QPointer>
 
 // KDE includes
 #include <kpassworddialog.h>
@@ -112,7 +115,7 @@ KSaneWidget::KSaneWidget(QWidget* parent)
     
     // Create the static UI
     // create the preview
-    d->m_previewViewer = new KSaneViewer(this);
+    d->m_previewViewer = new KSaneViewer(&(d->m_previewImg), this);
     connect(d->m_previewViewer, SIGNAL(newSelection(float, float, float, float)),
             d, SLOT(handleSelection(float, float, float, float)));
 
@@ -237,7 +240,9 @@ KSaneWidget::KSaneWidget(QWidget* parent)
 
 KSaneWidget::~KSaneWidget()
 {
-    closeDevice();
+    while (!closeDevice()) usleep(1000);
+    // wait for any thread to exit
+
     s_objectMutex.lock();
     s_objectCount--;
     if (s_objectCount <= 0) {
@@ -254,13 +259,14 @@ QString KSaneWidget::model() const {return d->m_model;}
 
 QString KSaneWidget::selectDevice(QWidget* parent)
 {
-  QString selected_name("");
-  KSaneDeviceDialog sel(parent);
+  QString selected_name;
+  QPointer<KSaneDeviceDialog> sel = new KSaneDeviceDialog(parent);
 
   // sel.setDefault(prev_backend); // set default scanner - perhaps application using libksane should remember that
-  if(sel.exec()) {
-      return selected_name = sel.getSelectedName();
+  if(sel->exec() == KDialog::Accepted) {
+      return selected_name = sel->getSelectedName();
   }
+  delete sel;
   return selected_name;
 }
 
@@ -345,7 +351,7 @@ bool KSaneWidget::openDevice(const QString &device_name)
 
         status = sane_open(device_name.toLatin1(), &d->m_saneHandle);
 
-        // store password in wallet on sucessful auth
+        // store password in wallet on successful authentication
         if(dlg->keepPassword() && status != SANE_STATUS_ACCESS_DENIED) {
             QMap<QString, QString> entry;
             entry["username"] = dlg->username().toUtf8();
@@ -376,7 +382,7 @@ bool KSaneWidget::openDevice(const QString &device_name)
     num_sane_options = *reinterpret_cast<SANE_Word*>(data.data());
 
     // read the rest of the options
-    for (i=1; i<num_sane_options; i++) {
+    for (i=1; i<num_sane_options; ++i) {
         switch (KSaneOption::otpionType(sane_get_option_descriptor(d->m_saneHandle, i))) {
             case KSaneOption::TYPE_DETECT_FAIL:
                 d->m_optList.append(new KSaneOption(d->m_saneHandle, i));
@@ -406,7 +412,8 @@ bool KSaneWidget::openDevice(const QString &device_name)
     }
 
     // do the connections of the option parameters
-    for (i=1; i<d->m_optList.size(); i++) {
+    for (i=1; i<d->m_optList.size(); ++i) {
+        //kDebug() << d->m_optList.at(i)->name();
         connect (d->m_optList.at(i), SIGNAL(optsNeedReload()), d, SLOT(optReload()));
         connect (d->m_optList.at(i), SIGNAL(valsNeedReload()), d, SLOT(scheduleValReload()));
     }
@@ -431,10 +438,10 @@ bool KSaneWidget::openDevice(const QString &device_name)
     d->m_btnFrame->setDisabled(false);
 
     // estimate the preview size and create an empty image
-    // this is done so that you can select scanarea without
+    // this is done so that you can select scan area without
     // having to scan a preview.
     d->updatePreviewSize();
-
+    QTimer::singleShot(0, d->m_previewViewer, SLOT(zoom2Fit()));
     return true;
 }
 
