@@ -36,6 +36,24 @@ class SearchPoint
         int offset_end;
 };
 
+/* text comparison functions */
+
+bool CaseInsensitiveCmpFn( const QStringRef & from, const QStringRef & to,
+                           int *fromLength, int *toLength )
+{
+    *fromLength = from.length();
+    *toLength = to.length();
+    return from.compare( to, Qt::CaseInsensitive ) == 0;
+}
+
+bool CaseSensitiveCmpFn( const QStringRef & from, const QStringRef & to,
+                           int *fromLength, int *toLength )
+{
+    *fromLength = from.length();
+    *toLength = to.length();
+    return from.compare( to, Qt::CaseSensitive ) == 0;
+}
+
 
 /*
   Rationale behind TinyTextEntity:
@@ -44,27 +62,56 @@ class SearchPoint
   we store the UTF-16 data and their length. This way, we save about
   4 int's wrt a QString, and we can create a new string from that
   raw data (that's the only penalty of that).
+  Even better, if the string we need to store has at most
+  MaxStaticChars characters, then we store those in place of the QChar*
+  that would be used (with new[] + free[]) for the data.
  */
 class TinyTextEntity
 {
+    static const int MaxStaticChars = sizeof( QChar * ) / sizeof( QChar );
+
     public:
         TinyTextEntity( const QString &text, const NormalizedRect &rect )
             : area( rect )
         {
             Q_ASSERT_X( !text.isEmpty(), "TinyTextEntity", "empty string" );
+            Q_ASSERT_X( sizeof( d ) == sizeof( QChar * ), "TinyTextEntity",
+                        "internal storage is wider than QChar*, fix it!" );
             length = text.length();
-            data = new QChar[ length ];
-            std::memcpy( data, text.constData(), length * sizeof( QChar ) );
+            switch ( length )
+            {
+#if QT_POINTER_SIZE >= 8
+                case 4:
+                    d.qc[3] = text.at( 3 ).unicode();
+                    // fall through
+                case 3:
+                    d.qc[2] = text.at( 2 ).unicode();
+                    // fall through
+#endif
+                case 2:
+                    d.qc[1] = text.at( 1 ).unicode();
+                    // fall through
+                case 1:
+                    d.qc[0] = text.at( 0 ).unicode();
+                    break;
+                default:
+                    d.data = new QChar[ length ];
+                    std::memcpy( d.data, text.constData(), length * sizeof( QChar ) );
+            }
         }
 
         ~TinyTextEntity()
         {
-            delete [] data;
+            if ( length > MaxStaticChars )
+            {
+                delete [] d.data;
+            }
         }
 
         inline QString text() const
         {
-            return QString::fromRawData( data, length );
+            return length <= MaxStaticChars ? QString::fromRawData( ( const QChar * )&d.qc[0], length )
+                                            : QString::fromRawData( d.data, length );
         }
 
         inline NormalizedRect transformedArea( const QMatrix &matrix ) const
@@ -79,7 +126,11 @@ class TinyTextEntity
     private:
         Q_DISABLE_COPY( TinyTextEntity )
 
-        QChar *data;
+        union
+        {
+            QChar *data;
+            ushort qc[MaxStaticChars];
+        } d;
         int length;
 };
 
@@ -175,7 +226,7 @@ RegularAreaRect * TextPage::textArea ( TextSelection * sel) const
 */
     RegularAreaRect * ret= new RegularAreaRect;
 
-    QMatrix matrix = d->m_page ? d->m_page->rotationMatrix() : QMatrix();
+    const QMatrix matrix = d->m_page ? d->m_page->rotationMatrix() : QMatrix();
 #if 0
     int it = -1;
     int itB = -1;
@@ -277,16 +328,16 @@ RegularAreaRect * TextPage::textArea ( TextSelection * sel) const
 #else
     NormalizedRect tmp;
 
-    NormalizedPoint startC = sel->start();
-    double startCx = startC.x;
-    double startCy = startC.y;
+    const NormalizedPoint startC = sel->start();
+    const double startCx = startC.x;
+    const double startCy = startC.y;
 
-    NormalizedPoint endC = sel->end();
-    double endCx = endC.x;
-    double endCy = endC.y;
+    const NormalizedPoint endC = sel->end();
+    const double endCx = endC.x;
+    const double endCy = endC.y;
 
     TextList::ConstIterator it = d->m_words.constBegin(), itEnd = d->m_words.constEnd();
-    MergeSide side = d->m_page ? (MergeSide)d->m_page->m_page->totalOrientation() : MergeRight;
+    const MergeSide side = d->m_page ? (MergeSide)d->m_page->m_page->totalOrientation() : MergeRight;
     for ( ; it != itEnd; ++it )
     {
         tmp = (*it)->area;
@@ -311,7 +362,7 @@ RegularAreaRect* TextPage::findText( int searchID, const QString &query, SearchD
         return 0;
     TextList::ConstIterator start;
     TextList::ConstIterator end;
-    QMap< int, SearchPoint* >::const_iterator sIt = d->m_searchPoints.constFind( searchID );
+    const QMap< int, SearchPoint* >::const_iterator sIt = d->m_searchPoints.constFind( searchID );
     if ( sIt == d->m_searchPoints.constEnd() )
     {
         // if no previous run of this search is found, then set it to start
@@ -352,13 +403,15 @@ RegularAreaRect* TextPage::findText( int searchID, const QString &query, SearchD
             break;
     };
     RegularAreaRect* ret = 0;
+    const TextComparisonFunction cmpFn = caseSensitivity == Qt::CaseSensitive
+                                       ? CaseSensitiveCmpFn : CaseInsensitiveCmpFn;
     if ( forward )
     {
-        ret = d->findTextInternalForward( searchID, query, caseSensitivity, start, end );
+        ret = d->findTextInternalForward( searchID, query, caseSensitivity, cmpFn, start, end );
     }
     else
     {
-        ret = d->findTextInternalBackward( searchID, query, caseSensitivity, start, end );
+        ret = d->findTextInternalBackward( searchID, query, caseSensitivity, cmpFn, start, end );
     }
     return ret;
 }
@@ -366,13 +419,14 @@ RegularAreaRect* TextPage::findText( int searchID, const QString &query, SearchD
 
 RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const QString &_query,
                                                              Qt::CaseSensitivity caseSensitivity,
+                                                             TextComparisonFunction comparer,
                                                              const TextList::ConstIterator &start,
                                                              const TextList::ConstIterator &end )
 {
-    QMatrix matrix = m_page ? m_page->rotationMatrix() : QMatrix();
+    const QMatrix matrix = m_page ? m_page->rotationMatrix() : QMatrix();
 
     RegularAreaRect* ret=new RegularAreaRect;
-    QString query = (caseSensitivity == Qt::CaseSensitive) ? _query : _query.toLower();
+    const QString query = (caseSensitivity == Qt::CaseSensitive) ? _query : _query.toLower();
 
     // j is the current position in our query
     // len is the length of the string in TextEntity
@@ -388,6 +442,7 @@ RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const Q
     {
         curEntity = *it;
         const QString &str = curEntity->text();
+	kDebug() << str;
         if ( !offsetMoved && ( it == start ) )
         {
             if ( m_searchPoints.contains( searchID ) )
@@ -405,10 +460,9 @@ RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const Q
             // we have equal (or less than) area of the query left as the length of the current 
             // entity
 
-            if ((caseSensitivity == Qt::CaseSensitive)
-                ? (str.mid(offset,min) != query.mid(j,min))
-                : (str.mid(offset,min).toLower() != query.mid(j,min))
-                )
+            int resStrLen = 0, resQueryLen = 0;
+            if ( !comparer( str.midRef( offset, min ), query.midRef( j, min ),
+                            &resStrLen, &resQueryLen ) )
             {
                     // we not have matched
                     // this means we do not have a complete match
@@ -430,15 +484,15 @@ RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const Q
                     // move the current position in the query
                     // to the position after the length of this string
                     // we matched
-                    // substract the length of the current entity from 
+                    // subtract the length of the current entity from 
                     // the left length of the query
 #ifdef DEBUG_TEXTPAGE
             kDebug(OkularDebug) << "\tmatched";
 #endif
                     haveMatch=true;
                     ret->append( curEntity->transformedArea( matrix ) );
-                    j+=min;
-                    queryLeft-=min;
+                    j += resStrLen;
+                    queryLeft -= resQueryLen;
                     if ( it_begin == TextList::ConstIterator() )
                     {
                         it_begin = it;
@@ -449,12 +503,12 @@ RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const Q
         if (haveMatch && queryLeft==0 && j==query.length())
         {
             // save or update the search point for the current searchID
-            if ( !m_searchPoints.contains( searchID ) )
+            QMap< int, SearchPoint* >::iterator sIt = m_searchPoints.find( searchID );
+            if ( sIt == m_searchPoints.end() )
             {
-                SearchPoint* newsp = new SearchPoint;
-                m_searchPoints.insert( searchID, newsp );
+                sIt = m_searchPoints.insert( searchID, new SearchPoint );
             }
-            SearchPoint* sp = m_searchPoints[ searchID ];
+            SearchPoint* sp = *sIt;
             sp->it_begin = it_begin;
             sp->it_end = it;
             sp->offset_begin = j;
@@ -464,10 +518,11 @@ RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const Q
         }
     }
     // end of loop - it means that we've ended the textentities
-    if ( m_searchPoints.contains( searchID ) )
+    const QMap< int, SearchPoint* >::iterator sIt = m_searchPoints.find( searchID );
+    if ( sIt != m_searchPoints.end() )
     {
-        SearchPoint* sp = m_searchPoints[ searchID ];
-        m_searchPoints.remove( searchID );
+        SearchPoint* sp = *sIt;
+        m_searchPoints.erase( sIt );
         delete sp;
     }
     delete ret;
@@ -476,10 +531,11 @@ RegularAreaRect* TextPagePrivate::findTextInternalForward( int searchID, const Q
 
 RegularAreaRect* TextPagePrivate::findTextInternalBackward( int searchID, const QString &_query,
                                                             Qt::CaseSensitivity caseSensitivity,
+                                                            TextComparisonFunction comparer,
                                                             const TextList::ConstIterator &start,
                                                             const TextList::ConstIterator &end )
 {
-    QMatrix matrix = m_page ? m_page->rotationMatrix() : QMatrix();
+    const QMatrix matrix = m_page ? m_page->rotationMatrix() : QMatrix();
 
     RegularAreaRect* ret=new RegularAreaRect;
     const QString query = (caseSensitivity == Qt::CaseSensitive) ? _query : _query.toLower();
@@ -531,10 +587,9 @@ RegularAreaRect* TextPagePrivate::findTextInternalBackward( int searchID, const
             // we have equal (or less than) area of the query left as the length of the current 
             // entity
 
-            if ((caseSensitivity == Qt::CaseSensitive)
-                ? (str.right(min) != query.mid(j-min+1,min))
-                : (str.right(min).toLower() != query.mid(j-min+1,min))
-                )
+            int resStrLen = 0, resQueryLen = 0;
+            if ( !comparer( str.rightRef( min ), query.midRef( j - min + 1, min ),
+                            &resStrLen, &resQueryLen ) )
             {
                     // we not have matched
                     // this means we do not have a complete match
@@ -556,15 +611,15 @@ RegularAreaRect* TextPagePrivate::findTextInternalBackward( int searchID, const
                     // move the current position in the query
                     // to the position after the length of this string
                     // we matched
-                    // substract the length of the current entity from 
+                    // subtract the length of the current entity from 
                     // the left length of the query
 #ifdef DEBUG_TEXTPAGE
                     kDebug(OkularDebug) << "\tmatched";
 #endif
                     haveMatch=true;
                     ret->append( curEntity->transformedArea( matrix ) );
-                    j-=min;
-                    queryLeft-=min;
+                    j -= resStrLen;
+                    queryLeft -= resQueryLen;
                     if ( it_begin == TextList::ConstIterator() )
                     {
                         it_begin = it;
@@ -594,7 +649,7 @@ RegularAreaRect* TextPagePrivate::findTextInternalBackward( int searchID, const
             --it;
     }
     // end of loop - it means that we've ended the textentities
-    QMap< int, SearchPoint* >::iterator sIt = m_searchPoints.find( searchID );
+    const QMap< int, SearchPoint* >::iterator sIt = m_searchPoints.find( searchID );
     if ( sIt != m_searchPoints.end() )
     {
         SearchPoint* sp = *sIt;
@@ -606,6 +661,11 @@ RegularAreaRect* TextPagePrivate::findTextInternalBackward( int searchID, const
 }
 
 QString TextPage::text(const RegularAreaRect *area) const
+{
+    return text(area, AnyPixelTextAreaInclusionBehaviour);
+}
+
+QString TextPage::text(const RegularAreaRect *area, TextAreaInclusionBehaviour b) const
 {
     if ( area && area->isNull() )
         return QString();
@@ -616,9 +676,20 @@ QString TextPage::text(const RegularAreaRect *area) const
     {
         for ( ; it != itEnd; ++it )
         {
-            if ( area->intersects( (*it)->area ) )
+            if (b == AnyPixelTextAreaInclusionBehaviour)
             {
-                ret += (*it)->text();
+                if ( area->intersects( (*it)->area ) )
+                {
+                    ret += (*it)->text();
+                }
+            }
+            else
+            {
+                NormalizedPoint center = (*it)->area.center();
+                if ( area->contains( center.x, center.y ) )
+                {
+                    ret += (*it)->text();
+                }
             }
         }
     }
